<?php

namespace ShopWP;

use ShopWP\Utils;
use ShopWP\Utils\Data as Utils_Data;
use ShopWP\Transients;

if (!defined('ABSPATH')) {
    exit();
}

class CPT
{
    public $DB_Settings_General;
    public $plugin_settings;

    public function __construct($DB_Settings_General, $plugin_settings)
    {
        $this->DB_Settings_General = $DB_Settings_General;
        $this->plugin_settings = $plugin_settings;
    }

    public function find_post_type_slug($type, $enable_default_pages)
    {
        if (!$enable_default_pages) {
            return $type;
        }

        $url = $this->plugin_settings['general']['url_' . $type];

        $parts = parse_url($url, PHP_URL_PATH);

        $slug = basename($parts);

        if (!$url || !$slug) {
            return $type;
        }

        return $slug;
    }

    public static function return_author_id()
    {
        if (get_current_user_id() === 0) {
            $author_id = 1;
        } else {
            $author_id = get_current_user_id();
        }

        return intval($author_id);
    }

    public static function set_post_id($post, $post_id)
    {
        $post->post_id = $post_id;

        return $post;
    }

    public static function add_props($item, $props)
    {
        foreach ($props as $key => $value) {
            $item->{$key} = $value;
        }

        return $item;
    }

    public static function add_props_to_items($items, $props)
    {
        return array_map(function ($item) use ($props) {
            return self::add_props($item, $props);
        }, $items);
    }

    public function post_type_products()
    {
        if (post_type_exists(SHOPWP_PRODUCTS_POST_TYPE_SLUG)) {
            return;
        }

        $enable_default_pages = $this->plugin_settings['general']['enable_default_pages'];

        $slug = $this->find_post_type_slug('products', $enable_default_pages);
        
        $rewrite_rules = [
            'slug' => $slug,
            'with_front' => false,
            'feeds' => true,
        ];

        $publicly_queryable = true;
        $exclude_from_search = false;

        $labels = [
            'name' => 'Products',
            'singular_name' => 'Product',
            'menu_name' => 'Products',
            'new_item' => 'Add New Product',
            'edit_item' => 'Edit Product',
            'not_found' => 'No Products found',
            'not_found_in_trash' => 'No Products found in trash',
        ];

        $args = [
            'label' => 'Products',
            'description' => 'Custom Post Type for Products',
            'labels' => $labels,
            'supports' => [
                'title',
                'page-attributes',
                'custom-fields',
                'thumbnail',
            ],
            'taxonomies' => ['shopify_tags', 'shopify_collections', 'shopify_vendor', 'shopify_product_type'],
            'hierarchical' => false,
            'public' => true,
            'show_ui' => true,
            'show_in_menu' => false,
            'menu_position' => 100,
            'menu_icon' => 'dashicons-megaphone',
            'show_in_admin_bar' => true,
            'show_in_nav_menus' => true,
            'can_export' => true,
            'show_in_rest' => true,
            'has_archive' => empty($enable_default_pages) ? false : true,
            'exclude_from_search' => $exclude_from_search,
            'publicly_queryable' => $publicly_queryable,
            'capability_type' => 'post',
            'rewrite' => $rewrite_rules,
            'capabilities' => [
                'create_posts' => false,
            ],
            'map_meta_cap' => true,
        ];

        register_post_type(
            SHOPWP_PRODUCTS_POST_TYPE_SLUG,
            apply_filters('shopwp_register_products_args', $args)
        );


        $fields = [
            'product_total_inventory',
            'product_total_variants',
        ];

        $args = array('show_in_rest' => true, 'type' => 'string', 'single' => true);

        foreach($fields as $field){
            register_post_meta('wps_products', $field, $args);
        }

    }

    public function post_type_collections()
    {
        if (post_type_exists(SHOPWP_COLLECTIONS_POST_TYPE_SLUG)) {
            return;
        }

        $enable_default_pages = $this->plugin_settings['general']['enable_default_pages'];

        $slug = $this->find_post_type_slug('collections', $enable_default_pages);

        $rewrite_rules = [
            'slug' => $slug,
            'with_front' => false,
            'feeds' => true,
        ];
        
        $publicly_queryable = true;
        $exclude_from_search = false;

        $labels = [
            'name' => 'Collections',
            'singular_name' => 'Collection',
            'menu_name' => 'Collections',
            'parent_item_colon' => 'Parent Item:',
            'new_item' => 'Add New Collection',
            'edit_item' => 'Edit Collection',
            'not_found' => 'No Collections found',
            'not_found_in_trash' => 'No Collections found in trash',
        ];

        $args = [
            'label' => 'Collections',
            'description' => 'Custom Post Type for Collections',
            'labels' => $labels,
            'supports' => [
                'title',
                'page-attributes',
                'custom-fields',
                'thumbnail',
            ],
            'hierarchical' => false,
            'public' => true,
            'show_ui' => true,
            'show_in_menu' => false,
            'menu_position' => 100,
            'menu_icon' => 'dashicons-megaphone',
            'show_in_admin_bar' => true,
            'show_in_nav_menus' => true,
            'can_export' => true,
            'has_archive' => empty($enable_default_pages) ? false : true,
            'show_in_rest' => true,
            'exclude_from_search' => $exclude_from_search,
            'publicly_queryable' => $publicly_queryable,
            'capability_type' => 'post',
            'rewrite' => $rewrite_rules,
            'capabilities' => [
                'create_posts' => false,
            ],
            'map_meta_cap' => true,
        ];

        register_post_type(
            SHOPWP_COLLECTIONS_POST_TYPE_SLUG,
            apply_filters('shopwp_register_collections_args', $args)
        );
    }

    public function post_type_shortcodes()
    {
        \register_post_type(
            'shopwp_shortcodes',
            [
                'label' => 'ShopWP Shortcodes',
                'description' => 'Allows you to create shortcodes using a visual builder for embedding ShopWP products',
                'labels' => [
                    'name' => 'ShopWP Shortcodes',
                    'singular_name' => 'Shortcode',
                    'menu_name' => 'shortcodes',
                    'new_item' => 'Add new shortcode',
                    'edit_item' => 'Edit shortcode',
                    'not_found' => 'No shortcode found',
                    'not_found_in_trash' => 'No shortcode found in trash',
                ],
                'supports' => [
                    'title',
                    'editor'
                ],
                'template' => [
                    [
                        'shopwp/products',
                        [],
                    ]
                ],
                'taxonomies' => [],
                'hierarchical' => false,
                'public' => true,
                'show_ui' => true,
                'show_in_menu' => false,
                'menu_position' => 100,
                'menu_icon' => 'dashicons-megaphone',
                'show_in_admin_bar' => true,
                'show_in_nav_menus' => true,
                'can_export' => false,
                'show_in_rest' => true,
                'has_archive' => false,
                'exclude_from_search' => true,
                'publicly_queryable' => false,
                'capability_type' => 'post',
                'rewrite' => [
                    'slug' => 'shopwp-shortcode',
                    'with_front' => false,
                    'feeds' => true,
                ],
                'map_meta_cap' => true,
            ]
        );

        \register_post_meta(
            'shopwp_shortcodes',
            'shopwp_shortcode_attributes',
            [
                'auth_callback' => '__return_true',
                'show_in_rest'  => true,
                'single'        => true,
                'type'          => 'string',
            ]
        );



    }

    public function register_custom_taxonomies()
    {

        if ( !taxonomy_exists('shopify_tags') ) {

            register_taxonomy('shopify_tags', ['wps_products'], [
                'label' => __('Shopify Tags', 'shopwp'),
                'hierarchical' => false,
                'public' => true,
                'description' => 'Shopify Tags',
                'show_admin_column' => true,
                'show_ui' => true,
                'show_in_rest' => true,
                'show_in_menu' => true,
                'rewrite' => ['slug' => 'product-tag']
            ]);

            register_taxonomy_for_object_type('shopify_tags', 'wps_products');
        }
        
        if ( !taxonomy_exists('shopify_vendor') ) {
            register_taxonomy('shopify_vendor', ['wps_products'], [
                'label' => __('Shopify Vendor', 'shopwp'),
                'hierarchical' => false,
                'public' => true,
                'description' => 'Shopify Vendor',
                'show_admin_column' => true,
                'show_ui' => true,
                'show_in_rest' => true,
                'show_in_menu' => true,
                'rewrite' => ['slug' => 'product-vendor']
            ]);

            register_taxonomy_for_object_type('shopify_vendor', 'wps_products');
        }

        if ( !taxonomy_exists('shopify_product_type') ) {
            register_taxonomy('shopify_product_type', ['wps_products'], [
                'label' => __('Shopify Product Type', 'shopwp'),
                'hierarchical' => false,
                'public' => true,
                'description' => 'Shopify Product Type',
                'show_admin_column' => true,
                'show_ui' => true,
                'show_in_rest' => true,
                'show_in_menu' => true,
                'rewrite' => ['slug' => 'product-type']
            ]);

            register_taxonomy_for_object_type('shopify_vendor', 'wps_products');
        }

        if ( !taxonomy_exists('shopify_collections') ) {

            register_taxonomy('shopify_collections', ['wps_products'], apply_filters('shopwp_register_shopify_collections_tax', [
                'label' => __('Shopify Collections', 'shopwp'),
                'hierarchical' => false,
                'public' => true,
                'description' => 'Shopify Collections',
                'show_admin_column' => true,
                'show_ui' => true,
                'show_in_rest' => true,
                'show_in_menu' => true,
                'rewrite' => ['slug' => 'collection'],
                'labels' => [
                    'name'                       => _x( 'Shopify Collections', 'taxonomy general name', 'shopwp' ),
                    'singular_name'              => _x( 'Shopify Collection', 'taxonomy singular name', 'shopwp' ),
                    'search_items'               => __( 'Search Collections', 'shopwp' ),
                    'popular_items'              => __( 'Popular Collections', 'shopwp' ),
                    'all_items'                  => __( 'All Collections', 'shopwp' ),
                    'parent_item'                => null,
                    'parent_item_colon'          => null,
                    'edit_item'                  => __( 'Edit Collection', 'shopwp' ),
                    'update_item'                => __( 'Update Collection', 'shopwp' ),
                    'add_new_item'               => __( 'Add New Collection', 'shopwp' ),
                    'new_item_name'              => __( 'New Collection Name', 'shopwp' ),
                    'separate_items_with_commas' => __( 'Separate collections with commas', 'shopwp' ),
                    'add_or_remove_items'        => __( 'Add or remove collections', 'shopwp' ),
                    'choose_from_most_used'      => __( 'Choose from the most used collections', 'shopwp' ),
                    'not_found'                  => __( 'No collections found.', 'shopwp' ),
                    'menu_name'                  => __( 'Shopify Collections', 'shopwp' ),
                ]
            ]));

            register_taxonomy_for_object_type('shopify_collections', 'wps_products');
        }

    }

    
 
    public function allowed_block_types_all( $allowed_blocks ) {

        $screen = get_current_screen();

        if  ( 'shopwp_shortcodes' == $screen->post_type ) {
            return [
                'shopwp/products'
            ];
        }

        return $allowed_blocks;
    
    }


    public function shortcodes_change_title_text( $title ){
        
        $screen = \get_current_screen();

        if  ( 'shopwp_shortcodes' == $screen->post_type ) {
            $title = 'Enter a shortcode name';
        }

        return $title;
    }

    public function add_shortcode_column_to_post_list($columns) {
        $custom_col_order = [
            'cb' => $columns['cb'],
            'title' => $columns['title'],
            'shortcode' => __( 'Shortcode', 'shopwp' ),
            'date' => $columns['date']
        ];
        
        return $custom_col_order;
    }


    public function add_shortcode_column_value_to_post_list($column_key, $post_id) {

        $post = \get_post($post_id);

        if (empty($post->post_content)) {
            echo "<p>No shortcode found</p>";
            return;
        }

        $onlyconsonants = \explode("<!-- wp:shopwp/products ", $post->post_content);

        if (empty($onlyconsonants)) {
            echo "<p>No shortcode found</p>";
            return;
        }

        $okok = \explode('"} -->', $onlyconsonants[1]);

        if (empty($okok)) {
            echo "<p>No shortcode found</p>";
            return;
        }

        $okok2 = \explode('{"settingsId":"', $okok[0]);

        if (empty($okok2)) {
            echo "<p>No shortcode found</p>";
            return;
        }

        $okok3 = \explode('","clientId"', $okok2[1]);

        if (empty($okok3)) {
            echo "<p>No shortcode found</p>";
            return;
        }

        $decoded_settings = base64_decode($okok3[0]);

        if (empty($decoded_settings)) {
            echo '<p>[wps_products]</p>';
        }

        $converted = Utils::convert_camel_to_underscore($decoded_settings);

        $product_settings = json_decode($converted, true);

        if (empty($product_settings)) {
            echo '<p>[wps_products]</p>';
        }        

        $shortcode_settings_string = '';

        foreach ($product_settings as $key => $value) {

            if ($key === 'query')  {
                continue;
            }

            if ($key === 'html_template_data') {

                $value = str_replace('<product_title />', '<ProductTitle />', $value);
                $value = str_replace('<product_pricing />', '<ProductPricing />', $value);
                $value = str_replace('<product_description />', '<ProductDescription />', $value);
                $value = str_replace('<product_images />', '<ProductImages />', $value);
                $value = str_replace('<product_buy_button />', '<ProductBuyButton />', $value);
                $value = str_replace('<reviews />', '<Reviews />', $value);

                $value = base64_encode($value);

            }

            if (is_array($value)) {

                $sub_data = '';

                foreach ($value as $v) {
                    $sub_data .= $v . ',';
                }

                $sub_data = rtrim($sub_data, ',');

                $shortcode_settings_string .= ' ' . $key . '="' . $sub_data . '"';

            } else {

                if (is_bool($value)) {
                    if ($value === false) {
                        $value = 'false';
                    } else {
                        $value = 'true';
                    }
                }

                $shortcode_settings_string .= ' ' . $key . '="' . $value . '"';
            }

        }

        $default_shortcode = '[wps_products' . $shortcode_settings_string . ']';

        if ($column_key == 'shortcode') {
            echo '<p>' . $default_shortcode . '</p>';
        }
    }
    

    public function init()
    {

        add_filter( 'allowed_block_types_all', [$this, 'allowed_block_types_all'] );
        add_filter( 'enter_title_here', [$this, 'shortcodes_change_title_text'] );

        $this->post_type_products();
        $this->post_type_collections();
        $this->register_custom_taxonomies();
        $this->post_type_shortcodes();

        add_filter('manage_shopwp_shortcodes_posts_columns', [$this, 'add_shortcode_column_to_post_list']);
        add_action('manage_shopwp_shortcodes_posts_custom_column', [$this, 'add_shortcode_column_value_to_post_list'], 10, 2);

    }
}
